@@ -150,6 +150,8 @@ urlpatterns = patterns('quant.django.apps.eui.views',
     (r'^persons/(?P<personName>%s)/approve/$' % quant.regexps.personName,
         'person.approve'),
 
+    (r'^persons/(?P<personName>%s)/apikey/$' % quant.regexps.personName,
+        'person.apikey'),
 
 
     #
@@ -166,6 +168,11 @@ urlpatterns = patterns('quant.django.apps.eui.views',
 
 
 
+    #
+    ##  API
+   
+    (r'^api/.*$',
+        'api.api'),
     #
     ## Specialized views
 
