@@ -4,7 +4,9 @@ class Book(SimpleObject):
 
     isUnique = False
 
-    title = String()
+    isTemporal = True
+
+    title = String(isTemporal=True)
     results = AggregatesMany('Result', key='id')
 
     def getLabelValue(self):
