@@ -12,7 +12,8 @@ def suite():
 class BookTestCase(TestCase):
 
     def setUp(self):
-        self.book = self.registry.books.read(title="My Book")
+        self.book = self.registry.books.values()[0]
+        self.failUnlessEqual(self.book.title, "My Book")
 
     def test_book(self):
         self.assertTrue(self.book)
