@@ -2,7 +2,9 @@ from quant.stats import NormalDistribution
 import scipy
 import math
 from quant.dsl import DslParser
+from quant.dsl import Date
 from quant.dsl import Market
+from quant.dsl import Fixing
 from quant.dsl import DslObject
 
 class Pricer(object):
@@ -36,6 +38,7 @@ class DerivativePricer(Pricer):
         self.annualRiskFreeRate = annualRiskFreeRate
         self.tolerance = tolerance
 
+# Todo: Change to not pass in optionValue0 (like Dsl works now).
     def calcDelta(self, optionValue0):
         #print self.__class__
         #print "estimated value: %s" % optionValue0
@@ -55,6 +58,7 @@ class DerivativePricer(Pricer):
         #print "estimated delta: %s" % optionDelta
         return optionDelta
 
+# Todo: Change to not pass in optionValue1 and optionDelta1 (like Dsl works now).
     def calcGamma(self, optionValue1, optionDelta1):
         lastPriceDelta = self.lastPrice * 0.001 * self.actualHistoricalVolatility / 100.0
         originalLastPrice = self.lastPrice
@@ -74,6 +78,7 @@ class DerivativePricer(Pricer):
         #print "estimated gamma: %s" % optionGamma
         return optionGamma
 
+# Todo: Change to not pass in optionValue0 (like Dsl works now).
     def calcVega(self, optionValue0):
         actualHistoricalVolatilityDelta = 0.001 * self.actualHistoricalVolatility
         originalActualHistoricalVolatility = self.actualHistoricalVolatility
@@ -259,57 +264,107 @@ class AmericanMonteCarlo(MonteCarlo):
 
 class DslMonteCarlo(Pricer):
 
+    pathCount = 500000
+
     def __init__(self, specification, image):
         self.specification = specification
         self.image = image
         self.dsl = None
+        self.allRvs = None
+        self.value = None
+        self.cache = {}
+
+    def calcValue(self):
+        return self.evaluateDsl(self.image)
+
+    def getPerturbedImage(self, market, factor):
+        return self.image.perturbMetric('last-price', market, factor)
+
+    def calcDelta(self, market, image=None, order=1):
+        if image == None:
+            image = self.image
+        if order == 0:
+            return self.evaluateDsl(image)
+        factor = self.calcPerturbationFactor(market)
+        perturbedImage = image.perturbMetric('last-price', market, factor)
+        delta = self.calcGradient(
+            self.calcDelta(market, image, order-1),
+            self.calcDelta(market, perturbedImage, order-1),
+            perturbedImage.getPerturbation()
+        )
+        return delta
+
+    def calcGamma(self, market):
+        return self.calcDelta(market, order=2)
+
+    def evaluateDsl(self, image):
+        if image not in self.cache:
+            dsl = self.getDsl()
+            allRvs = self.getAllRvs()
+            kwds = {
+                'image': image,
+                'allRvs': allRvs,
+                'presentTime': image.observationTime,
+                'interestRate': '2.5',
+            }
+            value = dsl.evaluate(**kwds)
+            if hasattr(value, 'mean'):
+                value = value.mean()
+            self.cache[image] = value
+        return self.cache[image]
 
     def getDsl(self):
         if self.dsl == None:
             self.dsl = DslParser().parse(self.specification)
         return self.dsl
 
-    def calcValue(self):
-        return self.getDsl().evaluate(image=self.image)
+    def calcPerturbationFactor(self, market):
+        actualHistoricalVolatility = self.image.getMetricValue('actual-historical-volatility', market)
+        return 0.001 * actualHistoricalVolatility / 100.0
+
+    def getAllRvs(self):
+        if self.allRvs == None:
+            self.allRvs = {}
+            markets = self.getMarkets()
+            times = self.getDates()
+            times = list(set(times))
+            times.sort()
+            for market in markets:
+                startTime = self.image.observationTime
+                marketRvs = {}
+                startRv = scipy.zeros(self.pathCount)
+                marketRvs[startTime] = startRv
+                for endTime in times:
+                    T = self.image.priceProcess.getDurationYears(startTime, endTime)
+                    draws = scipy.random.standard_normal(self.pathCount)
+                    endRv = startRv + scipy.sqrt(T) * draws
+                    marketRvs[endTime] = endRv
+                    startTime = endTime
+                    startRv = endRv
+                self.allRvs[market] = marketRvs
+        return self.allRvs
 
     def getMarkets(self, dslObject=None):
+        dslObjects = self.getDsl().findInstances(dslType=Market)
+        return [i.getDomainObject(self.image) for i in dslObjects]
+
+    def getDates(self, dslObject=None):
+        dslObjects = self.getDsl().findInstances(dslType=Date)
+        return [i.evaluate(image=self.image) for i in dslObjects]
+
+    def getFixings(self, dslObject=None):
+        return self.getDsl().findInstances(dslType=Fixing)
+
+    def getDslObjects(self, dslObject, dslType):
         if not dslObject:
             dslObject = self.getDsl()
-        markets = set()
+        dslObjects = set()
         for arg in dslObject.args:
-            if isinstance(arg, Market):
-                markets.add(arg.evaluate(image=self.image))
+            if isinstance(arg, dslType):
+                dslObjects.add(arg)
             if isinstance(arg, DslObject):
-                markets.update(self.getMarkets(arg))
-        return markets
-
-    def calcDelta(self, estimatedValue, market):
-        #print self.__class__
-        #print "estimated value: %s" % estimatedValue
-        actualHistoricalVolatility = self.image.getMetricValue('actual-historical-volatility', market)
-        factor = 0.001 * actualHistoricalVolatility / 100.0
-        perturbedImage = self.image.perturbMetric('last-price', market, factor)
-        perturbedValue = self.getDsl().evaluate(image=perturbedImage)
-        #print "perturbed value: %s" % perturbedValue
-        perturbation = perturbedImage.getPerturbation()
-        #print "perturbation: %s" % perturbation
-        estimatedDelta = self.calcGradient(estimatedValue, perturbedValue, perturbation)
-        #print "estimated delta: %s" % estimatedDelta
-        return estimatedDelta
-
-    def calcGamma(self, estimatedValue, estimatedDelta, market):
-        actualHistoricalVolatility = self.image.getMetricValue('actual-historical-volatility', market)
-        factor = -0.001 * actualHistoricalVolatility / 100.0
-        perturbedImage = self.image.perturbMetric('last-price', market, factor)
-        perturbedValue = self.getDsl().evaluate(image=perturbedImage)
-        #print "perturbed value: %s" % perturbedValue
-        perturbation = perturbedImage.getPerturbation()
-        #print "perturbation: %s" % perturbation
-        perturbedDelta = self.calcGradient(estimatedValue, perturbedValue, perturbation)
-        #print "perturbed delta: %s" % perturbedDelta
-        estimatedGamma = self.calcGradient(estimatedDelta, perturbedDelta, perturbation)
-        #print "estimated gamma: %s" % estimatedGamma
-        return estimatedGamma
+                dslObjects.update(self.getDslObjects(dslObject=arg, dslType=dslType))
+        return dslObjects
 
     def calcGradient(self, y1, y2, dx):
         return (y2 - y1) / dx
