@@ -5,6 +5,7 @@ import quant.priceprocess.test
 import quant.contracttype.test
 import quant.pricer.test
 import quant.leastsquarestest
+import quant.django.apps.eui.views.test
 
 def suite():
     suites = [
@@ -14,6 +15,7 @@ def suite():
         quant.contracttype.test.suite(),
         quant.pricer.test.suite(),
         quant.leastsquarestest.suite(),
+        quant.django.apps.eui.views.test.suite(),
     ]
     return unittest.TestSuite(suites)
 
